﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Reflection;
using System.Web;
using System.Web.Http;
using System.Web.Http.Controllers;
using System.Web.Http.Dispatcher;

namespace HeuristicLab.Services.WebApp.Controllers {
  public class WebAppHttpControllerSelector : DefaultHttpControllerSelector {
    private readonly HttpConfiguration configuration;
    private readonly IDictionary<string, IDictionary<string, HttpControllerDescriptor>> plugins;

    public WebAppHttpControllerSelector(HttpConfiguration configuration)
      : base(configuration) {
      this.configuration = configuration;
      plugins = new ConcurrentDictionary<string, IDictionary<string, HttpControllerDescriptor>>();
    }

    public override HttpControllerDescriptor SelectController(HttpRequestMessage request) {
      if (request == null) {
        throw new ArgumentNullException("request");
      }
      var parts = request.RequestUri.AbsolutePath.Split('/');
      int startIndex = parts.TakeWhile(part => part.ToLower() != "api").Count();
      if (parts.Length < startIndex + 2) {
        throw new ArgumentException("invalid request path");
      }
      string pluginName = parts[startIndex + 1];
      string controllerName = parts[startIndex + 2].ToLower();
      IDictionary<string, HttpControllerDescriptor> controllers;
      if (!plugins.TryGetValue(pluginName, out controllers)) {
        var assembly = Assembly.GetExecutingAssembly();
        try {
          if (pluginName != "App") {
            string directory = string.Format(@"{0}WebApp\plugins\{1}\", HttpRuntime.AppDomainAppPath, pluginName);
            string[] assemblies = Directory.GetFiles(directory, string.Format("HeuristicLab.Services.WebApp.{0}*.dll", pluginName));
            if (assemblies.Length < 1) {
              throw new ArgumentException("invalid plugin '{0}'", pluginName);
            }
            assembly = Assembly.Load(File.ReadAllBytes(assemblies.First()));
          }
          var assemblyTypes = assembly.GetTypes();
          var apiControllers = assemblyTypes.Where(c => typeof(ApiController).IsAssignableFrom(c)).ToList();
          if (apiControllers.Any()) {
            controllers = new ConcurrentDictionary<string, HttpControllerDescriptor>();
            foreach (var apiController in apiControllers) {
              var apiControllerName = apiController.Name.Remove(apiController.Name.Length - 10).ToLower();
              controllers.Add(apiControllerName, new HttpControllerDescriptor(configuration, apiControllerName, apiController));
            }
            plugins.Add(pluginName, controllers);
          }
        }
        catch (Exception) {
          throw new ArgumentException("error loading plugin '{0}'", pluginName);
        }
      }
      if (controllers == null) {
        throw new ArgumentException("invalid plugin '{0}'", pluginName);
      }
      HttpControllerDescriptor controller;
      controllers.TryGetValue(controllerName, out controller);
      return controller;
    }
  }
}