﻿var appName = 'app';
var appPath = 'WebApp';

var Route = function (name, view, controller) {
    this.name = name;
    this.view = view;
    this.controller = controller;
};

var Module = function (moduleName) {
    this.name = moduleName;
    this.parent = 'app';
    this.view = moduleName + '.cshtml';
    this.controller = this.parent + "." + this.name + ".ctrl";
    this.dependencies = [];
    this.routes = [];
    this.files = [];

    this.getModulePath = function() {
        return appPath + '/plugins/' + this.name + '/';
    };

    this.getFullModuleName = function () {
        if (moduleName.localeCompare(appName) == 0) {
            return this.name;
        }
        return this.parent + '.' + this.name;
    };

    this.getRouteName = function() {
        return '/' + this.name;
    }

    this.getServiceName = function() {
        return this.parent + "." + this.name + ".service";
    };

    this.getControllerName = function() {
        return this.parent + "." + this.name + ".ctrl";
    }

    this.getAngularModule = function() {
        return angular.module(this.getFullModuleName());
    };

    this.getFilePath = function(file) {
        return this.getModulePath() + file;
    };

    this.getViewUrl = function(view) {
        return 'plugin=' + this.name + '&view=' + view;
    };

    this.getFiles = function() {
        var module = this;
        var filesToLoad = [];
        module.files.forEach(function(file) {
            filesToLoad.push(module.getFilePath(file));
        });
        return filesToLoad;
    };

    this.getView = function() {
        return this.view;
    };

    this.getController = function() {
        return this.controller;
    };

    this.setView = function(view) {
        this.view = view;
    };

    this.setController = function(ctrl) {
        this.controller = ctrl;
    };

    this.configureFiles = function() {
        var module = this.getAngularModule();
        module.requires.push();
    };

    this.configureRoutes = function ($stateProvider) {
        var module = this;
        this.routes.forEach(function(route) {
            $stateProvider.state(module.name + route.name, {
                url: '/' + module.name + '/' + route.name,
                controller: route.controller,
                templateUrl: module.getViewUrl(route.view),
                cache: false,
                resolve: {
                    loadModule: ['$ocLazyLoad', 'cfpLoadingBar', function ($ocLazyLoad, cfpLoadingBar) {
                        cfpLoadingBar.start();
                        var retVal = module.loadModule($ocLazyLoad);
                        cfpLoadingBar.complete();
                        return retVal;
                    }]
                }
            });
        });
    };

    this.loadModule = function ($ocLazyLoad) {
        var module = this;
        var params = [];
        var lazyLoadingFiles = [];
        module.dependencies.forEach(function (dependency) {
            params.push(dependency);
        });
        module.files.forEach(function (file) {
            lazyLoadingFiles.push(module.getFilePath(file));
        });
        // load required files
        var lazyLoaded = null;
        if (!($ocLazyLoad === undefined || $ocLazyLoad === null)) {
            lazyLoaded = $ocLazyLoad.load(module.getFullModuleName());
        }
        // load angular module
        angular.module(this.getFullModuleName(), params);
        return lazyLoaded;
    };
};


var Menu = function () {
    this.sections = [];
    this.nextIndex = 0;

    this.getSection = function (name, index) {
        var length = this.sections.length;
        for (var i = 0; i < length; ++i) {
            var section = this.sections[i];
            if (section.name == name) {
                return section;
            }
        }
        if (index === undefined || index === null) {
            this.nextIndex++;
            index = this.nextIndex;
        } else {
            if (index > this.nextIndex) {
                this.nextIndex = index;
            }
        }
        var newSection = new Section(name, index);
        this.sections.push(newSection);
        return newSection;
    };

    this.getMenuEntries = function () {
        var entries = [];
        this.sections.sort(function (a, b) {
            return parseFloat(a.index) - parseFloat(b.index);
        });
        this.sections.forEach(function (section) {
            entries.push({name: section.name, isCategory: true, route: '?'});
            section.entries.forEach(function (entry) {
                entries.push(entry);
            });
        });
        return entries;
    };
};

var Section = function (name, index) {
    this.name = name;
    this.index = index;
    this.entries = [];

    this.addEntry = function (entry) {
        this.entries.push(entry);
    };
};

var app = new function () {
    this.modules = [];
    this.registerModule = function (moduleName) {
        var module = new Module(moduleName);
        if (moduleName.localeCompare(appName) != 0) {
            this.modules.push(module);
        }
        return module;
    };

    this.menu = new Menu();
    this.getMenu = function () {
        return this.menu;
    };
};